#include "windows.h"
#include "internal.h"
#include <math.h>

class chanmix : public GenericVideoFilter {   

	float r, g, b;

public:
 
	chanmix(PClip _child,	float _r, 	float _g,	float _b, IScriptEnvironment* env);
 
	~chanmix();
 
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
 
};

chanmix::chanmix(PClip _child,	float _r, 	float _g,	float _b, IScriptEnvironment* env) :
	GenericVideoFilter(_child), r(_r), g(_g), b(_b) {
 
	if (!vi.IsRGB24()) 
		env->ThrowError("chanmix: input to filter must be in RGB24 colourspace");

	if (r<0) r=0;
	if (g<0) r=0;
	if (b<0) r=0;
	if (r+b+g==0) 
		env->ThrowError("chanmix: r=b=g=0");


}

chanmix::~chanmix() {}

PVideoFrame __stdcall chanmix::GetFrame(int n, IScriptEnvironment* env) {

	PVideoFrame src = child->GetFrame(n, env);

	PVideoFrame dst = env->NewVideoFrame(vi);
	
	const unsigned char* srcp = src->GetReadPtr();

	unsigned char* dstp = dst->GetWritePtr();
	
	const int dst_pitch = dst->GetPitch();

	const int dst_width = dst->GetRowSize();

	const int dst_height = dst->GetHeight();

	const int src_pitch = src->GetPitch();
	const int src_width = src->GetRowSize();
	const int src_height = src->GetHeight();
	
	int w, h, res;
	for (h=0; h < src_height;h++) {       
		for (w = 0; w < src_width; w+=3) {   
			res = (int)floor((((float)*(srcp + w + 2)*r + (float)*(srcp + w + 1)*g + (float)*(srcp + w)*b)/(r+b+g))+0.5);

			*(dstp + w) = res;					
			*(dstp + w + 1) = res;     
			*(dstp + w + 2) = res;		
		}															
		
		srcp = srcp + src_pitch; 
		dstp = dstp + dst_pitch; 
	}
 	return dst;
}

AVSValue __cdecl Create_chanmix(AVSValue args, void* user_data, IScriptEnvironment* env) {

    return new chanmix(args[0].AsClip(),
		 args[1].AsFloat(1.0),	
		 args[2].AsFloat(1.0),	
		 args[3].AsFloat(1.0),	
		 env);  

}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("chanmix", "c[r]f[g]f[b]f", Create_chanmix, 0);

    return "`chanmix' channel-mixer plug-in";

}

